"use strict";

var _interopRequireWildcard3 = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.preparePlatform = preparePlatform;
exports.loadConfig = loadConfig;
exports.loadOlm = loadOlm;
exports.loadLanguage = loadLanguage;
exports.loadSkin = loadSkin;
exports.loadTheme = loadTheme;
exports.loadApp = loadApp;
exports.showError = showError;
exports.showIncompatibleBrowser = showIncompatibleBrowser;
exports._t = exports.rageshakePromise = void 0;

var _interopRequireWildcard2 = _interopRequireDefault(require("@babel/runtime/helpers/interopRequireWildcard"));

var _olm = _interopRequireDefault(require("olm/olm.wasm"));

var _olm2 = _interopRequireDefault(require("olm"));

var ReactDOM = _interopRequireWildcard3(require("react-dom"));

var React = _interopRequireWildcard3(require("react"));

var languageHandler = _interopRequireWildcard3(require("matrix-react-sdk/src/languageHandler"));

var _SettingsStore = _interopRequireDefault(require("matrix-react-sdk/src/settings/SettingsStore"));

var _ElectronPlatform = _interopRequireDefault(require("./platform/ElectronPlatform"));

var _WebPlatform = _interopRequireDefault(require("./platform/WebPlatform"));

var _PlatformPeg = _interopRequireDefault(require("matrix-react-sdk/src/PlatformPeg"));

var _SdkConfig = _interopRequireDefault(require("matrix-react-sdk/src/SdkConfig"));

var _theme = require("matrix-react-sdk/src/theme");

var _rageshakesetup = require("./rageshakesetup");

/*
Copyright 2015, 2016 OpenMarket Ltd
Copyright 2017 Vector Creations Ltd
Copyright 2018, 2019, 2020 New Vector Ltd
Copyright 2019 Michael Telatynski <7t3chguy@gmail.com>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
// @ts-ignore
const rageshakePromise = (0, _rageshakesetup.initRageshake)();
exports.rageshakePromise = rageshakePromise;

function preparePlatform() {
  if (window.ipcRenderer) {
    console.log("Using Electron platform");
    const plaf = new _ElectronPlatform.default();

    _PlatformPeg.default.set(plaf);
  } else {
    console.log("Using Web platform");

    _PlatformPeg.default.set(new _WebPlatform.default());
  }
}

async function loadConfig() {
  // XXX: We call this twice, once here and once in MatrixChat as a prop. We call it here to ensure
  // granular settings are loaded correctly and to avoid duplicating the override logic for the theme.
  //
  // Note: this isn't called twice for some wrappers, like the Jitsi wrapper.
  _SdkConfig.default.put((await _PlatformPeg.default.get().getConfig()) || {});
}

function loadOlm()
/*: Promise<void>*/
{
  /* Load Olm. We try the WebAssembly version first, and then the legacy,
   * asm.js version if that fails. For this reason we need to wait for this
   * to finish before continuing to load the rest of the app. In future
   * we could somehow pass a promise down to react-sdk and have it wait on
   * that so olm can be loading in parallel with the rest of the app.
   *
   * We also need to tell the Olm js to look for its wasm file at the same
   * level as index.html. It really should be in the same place as the js,
   * ie. in the bundle directory, but as far as I can tell this is
   * completely impossible with webpack. We do, however, use a hashed
   * filename to avoid caching issues.
   */
  return _olm2.default.init({
    locateFile: () => _olm.default
  }).then(() => {
    console.log("Using WebAssembly Olm");
  }).catch(e => {
    console.log("Failed to load Olm: trying legacy version", e);
    return new Promise((resolve, reject) => {
      const s = document.createElement('script');
      s.src = 'olm_legacy.js'; // XXX: This should be cache-busted too

      s.onload = resolve;
      s.onerror = reject;
      document.body.appendChild(s);
    }).then(() => {
      // Init window.Olm, ie. the one just loaded by the script tag,
      // not 'Olm' which is still the failed wasm version.
      return window.Olm.init();
    }).then(() => {
      console.log("Using legacy Olm");
    }).catch(e => {
      console.log("Both WebAssembly and asm.js Olm failed!", e);
    });
  });
}

async function loadLanguage() {
  const prefLang = _SettingsStore.default.getValue("language", null,
  /*excludeDefault=*/
  true);

  let langs = [];

  if (!prefLang) {
    languageHandler.getLanguagesFromBrowser().forEach(l => {
      langs.push(...languageHandler.getNormalizedLanguageKeys(l));
    });
  } else {
    langs = [prefLang];
  }

  try {
    await languageHandler.setLanguage(langs);
    document.documentElement.setAttribute("lang", languageHandler.getCurrentLanguage());
  } catch (e) {
    console.error("Unable to set language", e);
  }
}

async function loadSkin() {
  // Ensure the skin is the very first thing to load for the react-sdk. We don't even want to reference
  // the SDK until we have to in imports.
  console.log("Loading skin..."); // load these async so that its code is not executed immediately and we can catch any exceptions

  const [sdk, skin] = await Promise.all([Promise.resolve().then(() => (0, _interopRequireWildcard2.default)(require("matrix-react-sdk"))), Promise.resolve().then(() => (0, _interopRequireWildcard2.default)(require("../component-index")))]);
  sdk.loadSkin(skin);
  console.log("Skin loaded!");
}

async function loadTheme() {
  (0, _theme.setTheme)();
}

async function loadApp(fragParams
/*: {}*/
) {
  // load app.js async so that its code is not executed immediately and we can catch any exceptions
  const module = await Promise.resolve().then(() => (0, _interopRequireWildcard2.default)(require("./app")));
  window.matrixChat = ReactDOM.render(await module.loadApp(fragParams), document.getElementById('matrixchat'));
}

async function showError(title
/*: string*/
, messages
/*: string[]*/
) {
  const ErrorView = (await Promise.resolve().then(() => (0, _interopRequireWildcard2.default)(require("../async-components/structures/ErrorView")))).default;
  window.matrixChat = ReactDOM.render( /*#__PURE__*/React.createElement(ErrorView, {
    title: title,
    messages: messages
  }), document.getElementById('matrixchat'));
}

async function showIncompatibleBrowser(onAccept) {
  const CompatibilityView = (await Promise.resolve().then(() => (0, _interopRequireWildcard2.default)(require("../async-components/structures/CompatibilityView")))).default;
  window.matrixChat = ReactDOM.render( /*#__PURE__*/React.createElement(CompatibilityView, {
    onAccept: onAccept
  }), document.getElementById('matrixchat'));
}

const _t = languageHandler._t;
exports._t = _t;