"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.updateCheckStatusEnum = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _BasePlatform = _interopRequireDefault(require("matrix-react-sdk/src/BasePlatform"));

var _languageHandler = require("matrix-react-sdk/src/languageHandler");

var _getconfig = require("../getconfig");

var _favicon = _interopRequireDefault(require("../../favicon"));

/*
Copyright 2016 Aviral Dasgupta
Copyright 2016 OpenMarket Ltd
Copyright 2018, 2020 New Vector Ltd
Copyright 2019 Michael Telatynski <7t3chguy@gmail.com>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
const updateCheckStatusEnum = {
  CHECKING: 'CHECKING',
  ERROR: 'ERROR',
  NOTAVAILABLE: 'NOTAVAILABLE',
  DOWNLOADING: 'DOWNLOADING',
  READY: 'READY'
};
/**
 * Vector-specific extensions to the BasePlatform template
 */

exports.updateCheckStatusEnum = updateCheckStatusEnum;

class VectorBasePlatform extends _BasePlatform.default {
  constructor() {
    super();
    (0, _defineProperty2.default)(this, "_favicon", void 0);
    this.startUpdateCheck = this.startUpdateCheck.bind(this);
  }

  async getConfig()
  /*: Promise<{}>*/
  {
    return (0, _getconfig.getVectorConfig)();
  }

  getHumanReadableName()
  /*: string*/
  {
    return 'Vector Base Platform'; // no translation required: only used for analytics
  }
  /**
   * Delay creating the `Favicon` instance until first use (on the first notification) as
   * it uses canvas, which can trigger a permission prompt in Firefox's resist fingerprinting mode.
   * See https://github.com/vector-im/riot-web/issues/9605.
   */


  get favicon() {
    if (this._favicon) {
      return this._favicon;
    }

    return this._favicon = new _favicon.default();
  }

  _updateFavicon() {
    let bgColor = "#d00";
    let notif
    /*: string | number*/
    = this.notificationCount;

    if (this.errorDidOccur) {
      notif = notif || "×";
      bgColor = "#f00";
    }

    this.favicon.badge(notif, {
      bgColor
    });
  }

  setNotificationCount(count
  /*: number*/
  ) {
    if (this.notificationCount === count) return;
    super.setNotificationCount(count);

    this._updateFavicon();
  }

  setErrorStatus(errorDidOccur
  /*: boolean*/
  ) {
    if (this.errorDidOccur === errorDidOccur) return;
    super.setErrorStatus(errorDidOccur);

    this._updateFavicon();
  }
  /**
   * Begin update polling, if applicable
   */


  startUpdater() {}
  /**
   * Get a sensible default display name for the
   * device Vector is running on
   */


  getDefaultDeviceDisplayName()
  /*: string*/
  {
    return (0, _languageHandler._t)("Unknown device");
  }

}

exports.default = VectorBasePlatform;