"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _VectorBasePlatform = _interopRequireDefault(require("./VectorBasePlatform"));

var _BasePlatform = require("matrix-react-sdk/src/BasePlatform");

var _browserRequest = _interopRequireDefault(require("browser-request"));

var _dispatcher = _interopRequireDefault(require("matrix-react-sdk/src/dispatcher/dispatcher"));

var _languageHandler = require("matrix-react-sdk/src/languageHandler");

var _UpdateToast = require("matrix-react-sdk/src/toasts/UpdateToast");

var _actions = require("matrix-react-sdk/src/dispatcher/actions");

var _url = _interopRequireDefault(require("url"));

var _uaParserJs = _interopRequireDefault(require("ua-parser-js"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

const POKE_RATE_MS = 10 * 60 * 1000; // 10 min

class WebPlatform extends _VectorBasePlatform.default {
  constructor(...args) {
    super(...args);
    (0, _defineProperty2.default)(this, "runningVersion", null);
    (0, _defineProperty2.default)(this, "pollForUpdate", () => {
      return this._getVersion().then(ver => {
        if (this.runningVersion === null) {
          this.runningVersion = ver;
        } else if (this.runningVersion !== ver) {
          if (this.shouldShowUpdate(ver)) {
            (0, _UpdateToast.showToast)(this.runningVersion, ver);
          }

          return {
            status: _BasePlatform.UpdateCheckStatus.Ready
          };
        } else {
          (0, _UpdateToast.hideToast)();
        }

        return {
          status: _BasePlatform.UpdateCheckStatus.NotAvailable
        };
      }, err => {
        console.error("Failed to poll for update", err);
        return {
          status: _BasePlatform.UpdateCheckStatus.Error,
          detail: err.message || err.status ? err.status.toString() : 'Unknown Error'
        };
      });
    });
  }

  getHumanReadableName()
  /*: string*/
  {
    return 'Web Platform'; // no translation required: only used for analytics
  }
  /**
   * Returns true if the platform supports displaying
   * notifications, otherwise false.
   */


  supportsNotifications()
  /*: boolean*/
  {
    return Boolean(window.Notification);
  }
  /**
   * Returns true if the application currently has permission
   * to display notifications. Otherwise false.
   */


  maySendNotifications()
  /*: boolean*/
  {
    return window.Notification.permission === 'granted';
  }
  /**
   * Requests permission to send notifications. Returns
   * a promise that is resolved when the user has responded
   * to the request. The promise has a single string argument
   * that is 'granted' if the user allowed the request or
   * 'denied' otherwise.
   */


  requestNotificationPermission()
  /*: Promise<string>*/
  {
    // annoyingly, the latest spec says this returns a
    // promise, but this is only supported in Chrome 46
    // and Firefox 47, so adapt the callback API.
    return new Promise(function (resolve, reject) {
      window.Notification.requestPermission(result => {
        resolve(result);
      });
    });
  }

  displayNotification(title
  /*: string*/
  , msg
  /*: string*/
  , avatarUrl
  /*: string*/
  , room
  /*: Room*/
  ) {
    const notifBody = {
      body: msg,
      tag: "vector",
      silent: true // we play our own sounds

    };
    if (avatarUrl) notifBody['icon'] = avatarUrl;
    const notification = new window.Notification(title, notifBody);

    notification.onclick = function () {
      _dispatcher.default.dispatch({
        action: 'view_room',
        room_id: room.roomId
      });

      window.focus();
      notification.close();
    };
  }

  _getVersion()
  /*: Promise<string>*/
  {
    // We add a cachebuster to the request to make sure that we know about
    // the most recent version on the origin server. That might not
    // actually be the version we'd get on a reload (particularly in the
    // presence of intermediate caching proxies), but still: we're trying
    // to tell the user that there is a new version.
    return new Promise(function (resolve, reject) {
      (0, _browserRequest.default)({
        method: "GET",
        url: "version",
        qs: {
          cachebuster: Date.now()
        }
      }, (err, response, body) => {
        if (err || response.status < 200 || response.status >= 300) {
          if (err === null) err = {
            status: response.status
          };
          reject(err);
          return;
        }

        const ver = body.trim();
        resolve(ver);
      });
    });
  }

  getAppVersion()
  /*: Promise<string>*/
  {
    if (this.runningVersion !== null) {
      return Promise.resolve(this.runningVersion);
    }

    return this._getVersion();
  }

  startUpdater() {
    this.pollForUpdate();
    setInterval(this.pollForUpdate, POKE_RATE_MS);
  }

  async canSelfUpdate()
  /*: Promise<boolean>*/
  {
    return true;
  }

  startUpdateCheck() {
    super.startUpdateCheck();
    this.pollForUpdate().then(updateState => {
      _dispatcher.default.dispatch(_objectSpread({
        action: _actions.Action.CheckUpdates
      }, updateState));
    });
  }

  installUpdate() {
    window.location.reload(true);
  }

  getDefaultDeviceDisplayName()
  /*: string*/
  {
    // strip query-string and fragment from uri
    const u = _url.default.parse(window.location.href);

    u.protocol = "";
    u.search = "";
    u.hash = ""; // Remove trailing slash if present

    u.pathname = u.pathname.replace(/\/$/, "");
    let appName = u.format(); // Remove leading slashes if present

    appName = appName.replace(/^\/\//, ""); // `appName` is now in the format `riot.im/develop`.

    const ua = new _uaParserJs.default();
    const browserName = ua.getBrowser().name || "unknown browser";
    let osName = ua.getOS().name || "unknown OS"; // Stylise the value from the parser to match Apple's current branding.

    if (osName === "Mac OS") osName = "macOS";
    return (0, _languageHandler._t)('%(appName)s (%(browserName)s, %(osName)s)', {
      appName,
      browserName,
      osName
    });
  }

  screenCaptureErrorString()
  /*: string | null*/
  {
    // it won't work at all if you're not on HTTPS so whine whine whine
    if (window.location.protocol !== "https:") {
      return (0, _languageHandler._t)("You need to be using HTTPS to place a screen-sharing call.");
    }

    return null;
  }

  reload() {
    // forceReload=false since we don't really need new HTML/JS files
    // we just need to restart the JS runtime.
    window.location.reload(false);
  }

}

exports.default = WebPlatform;